@@ -126,11 +126,11 @@ class WriteOptions {
   }
 
  private:
-  void SetBit(const gpr_int32 mask) { flags_ |= mask; }
+  void SetBit(const gpr_uint32 mask) { flags_ |= mask; }
 
-  void ClearBit(const gpr_int32 mask) { flags_ &= ~mask; }
+  void ClearBit(const gpr_uint32 mask) { flags_ &= ~mask; }
 
-  bool GetBit(const gpr_int32 mask) const { return flags_ & mask; }
+  bool GetBit(const gpr_uint32 mask) const { return (flags_ & mask) != 0; }
 
   gpr_uint32 flags_;
 };
@@ -248,8 +248,7 @@ class CallOpRecvMessage {
       if (*status) {
         got_message = true;
         *status = SerializationTraits<R>::Deserialize(recv_buf_, message_,
-                                                      max_message_size)
-                      .ok();
+                                                      max_message_size).ok();
       } else {
         got_message = false;
         grpc_byte_buffer_destroy(recv_buf_);
