@@ -40,6 +40,12 @@
 extern "C" {
 #endif
 
+typedef struct gpr_allocation_functions {
+  void *(*malloc_fn)(size_t size);
+  void *(*realloc_fn)(void *ptr, size_t size);
+  void (*free_fn)(void *ptr);
+} gpr_allocation_functions;
+
 /* malloc, never returns NULL */
 void *gpr_malloc(size_t size);
 /* free */
@@ -51,6 +57,14 @@ void *gpr_malloc_aligned(size_t size, size_t alignment_log);
 /* free memory allocated by gpr_malloc_aligned */
 void gpr_free_aligned(void *ptr);
 
+/** Request the family of allocation functions in \a functions be used. NOTE
+ * that this request will be honored in a *best effort* basis and that no
+ * guarantees are made about the default functions (eg, malloc) being called. */
+void gpr_set_allocation_functions(gpr_allocation_functions functions);
+
+/** Return the family of allocation functions currently in effect. */
+gpr_allocation_functions gpr_get_allocation_functions();
+
 #ifdef __cplusplus
 }
 #endif
