@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/usr/bin/env bash
 # Copyright 2015, Google Inc.
 # All rights reserved.
 #
@@ -31,8 +31,6 @@
 # This script is invoked by Jenkins and triggers a test run based on
 # env variable settings.
 #
-# Bootstrap into bash
-[ -z $1 ] && exec bash $0 bootstrapped
 # Setting up rvm environment BEFORE we set -ex.
 [[ -s /etc/profile.d/rvm.sh ]] && . /etc/profile.d/rvm.sh
 # To prevent cygwin bash complaining about empty lines ending with \r
@@ -56,50 +54,8 @@ if [ "$platform" == "linux" ]
 then
   echo "building $language on Linux"
 
-  cd `dirname $0`/../..
-  git_root=`pwd`
-  cd -
+  ./tools/run_tests/run_tests.py --use_docker -t -l $language -c $config -x report.xml $@ || true
 
-  mkdir -p /tmp/ccache
-
-  # Use image name based on Dockerfile checksum
-  DOCKER_IMAGE_NAME=grpc_jenkins_slave$docker_suffix_`sha1sum tools/jenkins/grpc_jenkins_slave/Dockerfile | cut -f1 -d\ `
-
-  # Make sure docker image has been built. Should be instantaneous if so.
-  docker build -t $DOCKER_IMAGE_NAME tools/jenkins/grpc_jenkins_slave$docker_suffix
-
-  # Create a local branch so the child Docker script won't complain
-  git branch jenkins-docker
-
-  # Make sure the CID file is gone.
-  rm -f docker.cid
-
-  # Run tests inside docker
-  docker run \
-    -e "config=$config" \
-    -e "language=$language" \
-    -e "arch=$arch" \
-    -e "GRPC_ZOOKEEPER_SERVER_TEST=grpc-jenkins-master:2181" \
-    -e CCACHE_DIR=/tmp/ccache \
-    -i \
-    -v "$git_root:/var/local/jenkins/grpc" \
-    -v /tmp/ccache:/tmp/ccache \
-    --cidfile=docker.cid \
-    $DOCKER_IMAGE_NAME \
-    bash -l /var/local/jenkins/grpc/tools/jenkins/docker_run_jenkins.sh || DOCKER_FAILED="true"
-
-  DOCKER_CID=`cat docker.cid`
-  # forcefully kill the instance if it's still running, otherwise
-  # continue 
-  # (failure to kill something that's already dead => things are dead)
-  docker kill $DOCKER_CID || true
-  docker cp $DOCKER_CID:/var/local/git/grpc/report.xml $git_root
-  # TODO(ctiller): why?
-  sleep 4
-  docker rm $DOCKER_CID || true
-elif [ "$platform" == "interop" ]
-then
-  python tools/run_tests/run_interops.py --language=$language
 elif [ "$platform" == "windows" ]
 then
   echo "building $language on Windows"
@@ -107,24 +63,27 @@ then
   # Prevent msbuild from picking up "platform" env variable, which would break the build
   unset platform
 
-  # TODO(jtattermusch): integrate nuget restore in a nicer way.
-  /cygdrive/c/nuget/nuget.exe restore vsprojects/grpc.sln
-  /cygdrive/c/nuget/nuget.exe restore src/csharp/Grpc.sln
-
-  python tools/run_tests/run_tests.py -t -l $language -x report.xml || true
+  python tools/run_tests/run_tests.py -t -l $language -c $config -x report.xml $@ || true
 
 elif [ "$platform" == "macos" ]
 then
   echo "building $language on MacOS"
 
-  ./tools/run_tests/run_tests.py -t -l $language -c $config -x report.xml || true
+  ./tools/run_tests/run_tests.py -t -l $language -c $config -x report.xml $@ || true
 
 elif [ "$platform" == "freebsd" ]
 then
   echo "building $language on FreeBSD"
 
-  MAKE=gmake ./tools/run_tests/run_tests.py -t -l $language -c $config -x report.xml || true
+  MAKE=gmake ./tools/run_tests/run_tests.py -t -l $language -c $config -x report.xml $@ || true
+
 else
   echo "Unknown platform $platform"
   exit 1
 fi
+
+if [ ! -e reports/index.html ]
+then
+  mkdir -p reports
+  echo 'No reports generated.' > reports/index.html
+fi
